@@ -14,6 +14,9 @@ def main():
     etld_lib_spawn_etl.spawned_process_sleep_time = 5               # check every n seconds for spawned_process.is_alive()
     etld_lib_spawn_etl.spawned_process_count_to_status_update = 11  # Every n checks print status spawned_process.is_alive()
 
+    etld_lib_spawn_etl.etl_dir_to_monitor = etld_lib_config.qetl_user_data_dir
+    etld_lib_spawn_etl.etl_files_max_time_between_changes = 300
+
     etld_lib_spawn_etl.target_module_to_run = etl_host_list.host_list_etl_workflow
     etld_lib_spawn_etl.target_method_in_module_to_run = "host_list_etl_workflow"
     etld_lib_spawn_etl.etl_main()
