@@ -9,7 +9,7 @@ import shutil
 from qualys_etl.etld_lib import etld_lib_config as etld_lib_config
 from qualys_etl.etld_lib import etld_lib_functions as etld_lib_functions
 from qualys_etl.etld_lib import etld_lib_sqlite_tables as etld_lib_sqlite_tables
-from qualys_etl.etld_lib import etld_lib_date_time_functions as api_datetime
+from qualys_etl.etld_lib import etld_lib_datetime as api_datetime
 import qualys_etl.etld_host_list_detection.host_list_detection_process_00_extract as host_list_detection_extract
 
 global host_list_records
