@@ -11,7 +11,7 @@ from pathlib import Path
 import qualys_etl.etld_lib.etld_lib_functions as etld_lib_functions
 import qualys_etl.etld_lib.etld_lib_config as etld_lib_config
 import qualys_etl.etld_lib.etld_lib_credentials as etld_lib_credentials
-import qualys_etl.etld_lib.etld_lib_date_time_functions as api_datetime
+import qualys_etl.etld_lib.etld_lib_datetime as api_datetime
 
 global qetl_user_home
 global found_api_credentials
@@ -34,6 +34,14 @@ def validate_api_fqdn_server(api_fqdn_server):
         return True
 
 
+def validate_gateway_fqdn_server(gateway_fqdn_server):
+    gateway_fqdn_server_match = re.fullmatch(r"^.*gateway\..*$", gateway_fqdn_server)  # Matches gateway
+    if gateway_fqdn_server_match is None:
+        return False
+    else:
+        return True
+
+
 def validate_password(password):
     password_match = re.fullmatch(r"[ ]+", password)
     if password_match is None:
@@ -59,6 +67,25 @@ def update_api_fqdn_server(cred):
     pass
 
 
+def update_gateway_fqdn_server(cred):
+    if cred.get('api_fqdn_server') in etld_lib_credentials.platform_url.keys():
+        cred['gateway_fqdn_server'] = etld_lib_credentials.platform_url.get(cred.get('gateway_fqdn_server'))
+    print(f"Current gateway_fqdn_server: {cred.get('gateway_fqdn_server')}")
+    response = input(f"Update gateway_fqdn_server? ( yes or no ): ")
+    if response in 'yes':
+        while True:
+            new_gateway_fqdn_server = input(f"Enter new gateway_fqdn_server: ")
+            new_gateway_fqdn_server = re.sub("https://", '', new_gateway_fqdn_server)
+            new_gateway_fqdn_server = re.sub("http://", '', new_gateway_fqdn_server)
+            new_gateway_fqdn_server = re.sub("/", '', new_gateway_fqdn_server)
+            if validate_gateway_fqdn_server(new_gateway_fqdn_server):
+                break
+            else:
+                print(f"Not valid gateway FQDN, retry.")
+        cred['gateway_fqdn_server'] = new_gateway_fqdn_server
+    pass
+
+
 def update_username(cred):
     time.sleep(1)
     print(f"\n\nCurrent username: {cred.get('username')} in config: {etld_lib_credentials.cred_file}")
@@ -98,12 +125,14 @@ def if_update_credentials(first_time=False):
         update_password(credentials)
         old_cred = etld_lib_credentials.get_cred()
         if old_cred == credentials:
-            print(f"No changes to qualys username, password or api_fqdn_server.")
+            print(f"No changes to qualys username, password, api_fqdn_server or gateway_fqdn_server.")
         else:
             etld_lib_credentials.update_cred(credentials)
             new_credentials = etld_lib_credentials.get_cred()
             etld_lib_functions.logger.info(f"credentials updated.  username: {new_credentials.get('username')} "
-                                 f"api_fqdn_server: {new_credentials.get('api_fqdn_server')} ")
+                                           f"api_fqdn_server: {new_credentials.get('api_fqdn_server')} "
+                                           f"gateway_fqdn_server: {new_credentials.get('gateway_fqdn_server')} "
+                                           f"")
             print(f"You have updated your credentials.")
             print(f"  Qualys Username: {new_credentials.get('username')}")
             print(f"  Qualys api_fqdn_server: {new_credentials.get('api_fqdn_server')}\n")
@@ -124,6 +153,14 @@ def start_etl_host_list():
     print(f"End      etl_host_list.  For results see:  {etld_lib_config.host_list_log_file}")
 
 
+def start_etl_asset_inventory():
+    import qualys_etl.etld_asset_inventory.etld_00_spawn_workflow_manager_asset_inventory \
+        as etl_asset_inventory_spawn_from_qetl_manage_user
+    print(f"Starting etl_asset_inventory.  For progress see: {etld_lib_config.asset_inventory_log_file}")
+    etl_asset_inventory_spawn_from_qetl_manage_user.main()
+    print(f"End      etl_asset_inventory.  For progress see: {etld_lib_config.asset_inventory_log_file}")
+
+
 def start_etl_host_list_detection():
     import qualys_etl.etld_host_list_detection.etld_00_spawn_workflow_manager_host_list_detection \
        as etl_host_list_detection_spawn_from_qetl_manage_user
@@ -155,10 +192,13 @@ def if_execute_etl_module():
         ## TODO add fcntl lock on etl_knowledgebase and etl_host_list so they are not run concurrently with etl_host_list_detection.
         ## TODO add fcntl lock check logic to stop running if host list or knowledgebase are already executing.
         start_etl_host_list_detection()
+    elif command_line_arguments.execute_etl_module == 'etl_asset_inventory':
+        start_etl_asset_inventory()
     elif command_line_arguments.execute_etl_module is None:
         pass
     else:
-        etld_lib_functions.logger.info(f"Invalid Option: {str(command_line_arguments.execute_etl_module)}, retry when ready.")
+        etld_lib_functions.logger.info(f"Invalid Option: {str(command_line_arguments.execute_etl_module)}, "
+                                       f"retry when ready.")
         print(f"\nInvalid Option: {str(command_line_arguments.execute_etl_module)}, retry when ready")
         print(f"Options are: -e etl_knowledgebase or -e etl_host_list or -e etl_host_list_detection")
         exit(1)
@@ -180,6 +220,22 @@ def if_qualys_test_login(first_time=False):
         etld_lib_functions.main(log_level=logging.WARN, my_logger_prog_name='qetl_manage_user')
 
 
+def if_qualys_test_gateway_login(first_time=False):
+    if command_line_arguments.test is not None or first_time is not False:
+        etld_lib_functions.main(log_level=logging.INFO, my_logger_prog_name='qetl_manage_user')
+        etld_lib_config.main()
+        etld_lib_credentials.main()
+        credentials = etld_lib_credentials.get_cred()
+        print(f"Qualys Login Test for {credentials.get('username')} "
+              f"at gateway_fqdn_server: {credentials.get('gateway_fqdn_server')}\n")
+        etld_lib_credentials.test_gateway_auth()
+        if etld_lib_credentials.login_failed is not True:
+            print(f"Testing Qualys Login for {credentials.get('username')} "
+                  f"Succeeded at {credentials.get('api_fqdn_server')}\n"
+                  f"    with HTTPS Return Code: {etld_lib_credentials.http_return_code}.")
+        etld_lib_functions.main(log_level=logging.WARN, my_logger_prog_name='qetl_manage_user')
+
+
 def if_print_report():
     if command_line_arguments.report is not None:
         print(f"Report on user: {etld_lib_config.qetl_user_home_dir}")
@@ -209,6 +265,7 @@ def help_message(notes):
       -e etl module name,     --execute_etl_module module name
                                   Ex.  -e etl_knowledgebase or -e etl_host_list or -e etl_host_list_detection
       -d YYMMDDThh:mm:ssZ,    --datetime        YYYY-MM-DDThh:mm:ssZ UTC. Get All Data On or After Date. 
+                                                Ex. 1970-01-01T00:00:00Z acts as flag to obtain all data.
       -c, --credentials       update qualys api user credentials: qualys username, password or api_fqdn_server
       -t, --test              test qualys credentials
       -l, --logs              detailed logs sent to stdout
@@ -265,7 +322,7 @@ def get_command_line_arguments(args=None):
     parser.add_argument('-u', '--qetl_user_home_dir', default=None, help="Please enter -u option")
     parser.add_argument('-e', '--execute_etl_module', default=None, help='Execute etl_knowledgebase, etl_host_list or etl_host_list_detection')
     parser.add_argument('-d', '--datetime', default=None,
-                        help='YYYY-MM-DDThh:mm:ssZ UTC. Get All Data On or After Date')
+                        help='YYYY-MM-DDThh:mm:ssZ UTC. Get All Data On or After Date.  Ex. 1970-01-01T00:00:00Z acts as flag to obtain all data.')
     parser.add_argument('-c', '--credentials', default=None, action="store_true",
                         help='update qualys api user credentials: qualys uername, password or api_fqdn_server')
     parser.add_argument('-t', '--test', default=None, action="store_true", help='test qualys credentials')
@@ -309,10 +366,14 @@ def setup_qetl_user_home_dir_environment():
             time.sleep(1)
             print(f"\nqetl_user_home_dir created: {etld_lib_config.qetl_user_home_dir}")
             if_update_credentials(first_time=True)
-            response = input(f"\nWould you like to test login/logout of Qualys? ( yes or no ): ")
+            response = input(f"\nWould you like to test login/logout of Qualys API? ( yes or no ): ")
             if response == 'yes':
                 print("")
                 if_qualys_test_login(first_time=True)
+            # response = input(f"\nWould you like to test login/logout of Qualys Gateway? ( yes or no ): ")
+            # if response == 'yes':
+            #     print("")
+            #     if_qualys_test_gateway_login(first_time=True)
             print(f"\nThank you, exiting.\n")
             exit(0)
         else:
